let cmdtable = (hashtbl__new 19 : (string, action list) hashtbl__t);;

let def name action =
  hashtbl__add cmdtable name action;;

let find_macro name =
  try
    hashtbl__find cmdtable name
  with Not_found ->
    prerr_string "Unknown macro: "; prerr_endline name; [];;

(* General LaTeX macros *)

def "\\part"
    [Print "<H0>"; Print_arg; Print "</H0>\n"];
def "\\chapter"
    [Print "<H1>"; Print_arg; Print "</H1>\n"];
def "\\chapter*"
    [Print "<H1>"; Print_arg; Print "</H1>\n"];
def "\\section"
    [Print "<H2>"; Print_arg; Print "</H2>\n"];
def "\\section*"
    [Print "<H2>"; Print_arg; Print "</H2>\n"];
def "\\subsection"
    [Print "<H3>"; Print_arg; Print "</H3>\n"];
def "\\subsection*"
    [Print "<H3>"; Print_arg; Print "</H3>\n"];
def "\\subsubsection"
    [Print "<H4>"; Print_arg; Print "</H4>\n"];
def "\\subsubsection*"
    [Print "<H4>"; Print_arg; Print "</H4>\n"];
def "\\paragraph"
    [Print "<H5>"; Print_arg; Print "</H5>\n"];
def "\\begin{alltt}" [Print "<pre>"];
def "\\end{alltt}" [Print "</pre>"];
def "\\begin{itemize}" [Print "<p><ul>"];
def "\\end{itemize}" [Print "</ul>"];
def "\\begin{enumerate}" [Print "<p><ol>"];
def "\\end{enumerate}" [Print "</ol>"];
def "\\begin{description}" [Print "<p><dl>"];
def "\\end{description}" [Print "</dl>"];
def "\\begin{center}" [Print "<blockquote>"];
def "\\end{center}" [Print "</blockquote>"];
def "\\smallskip" [];
def "\\medskip" [];
def "\\bigskip" [];
def "\\markboth" [Skip_arg; Skip_arg];
def "\\ldots" [Print "..."];
def "\\ " [Print " "];
def "\\{" [Print "{"];
def "\\}" [Print "}"];
def "\\/" [];
def "\\newpage" [];
def "\\label" [Print "<A name=\""; Print_arg; Print "\"></A>"];
def "\\ref" [Print "<A href=\"#"; Print_arg; Print "\">(ref)</A>"];
def "\\index" [Skip_arg];
def "\\oe" [Print "oe"];
def "\\&" [Print "&amp;"];
def "\\_" [Print "_"];
def "\\leq" [Print "&lt;="];
def "\\geq" [Print "&gt;="];
def "\\hbox" [Print_arg];
def "\\copyright" [Print "(c)"];
def "\\noindent" [];
def "\\begin{flushleft}" [Print "<blockquote>"];
def "\\end{flushleft}" [Print "</blockquote>"];
def "\\\\" [Print "<br>"];
def "\\(" [Print "<I>"];
def "\\)" [Print "</I>"];
def "\\cite" [Raw_arg(function r -> 
  print_string ("<A HREF=\"biblio.html#" ^ r ^ "\">[" ^ r ^ "]</A>"))];
def "\\begin{htmlonly}" [];
def "\\end{htmlonly}" [];
def "\\begin{thebibliography}" [Print "<H2>References</H2>\n<dl>\n"; Skip_arg];
def "\\end{thebibliography}" [Print "</dl>"];
def "\\bibitem" [Raw_arg(function r ->
  print_string "<dt><A name=\""; print_string r; print_string "\">[";
  print_string r; print_string "]</A>\n";
  print_string "<dd>")];
def "\\newblock" [Print "<br>"];
def "\\'" [Raw_arg(function "e" -> print_char ``
                          | "E" -> print_char ``
                          | s   -> print_string s)];
def "\\`" [Raw_arg(function "e" -> print_char ``
                          | "E" -> print_char ``
                          | "a" -> print_char ``
                          | "A" -> print_char ``
                          | "u" -> print_char ``
                          | "U" -> print_char ``
                          | s   -> print_string s)];
def "\\^" [Raw_arg(function "a" -> print_char ``
                          | "A" -> print_char ``
                          | "e" -> print_char ``
                          | "E" -> print_char ``
                          | "i" -> print_char ``
                          | "I" -> print_char ``
                          | "o" -> print_char ``
                          | "O" -> print_char ``
                          | "u" -> print_char ``
                          | "U" -> print_char ``
                          | s   -> print_string s)];
def "\\\"" [Raw_arg(function "e" -> print_char ``
                          | "E" -> print_char ``
                          | "i" -> print_char ``
                          | "I" -> print_char ``
                          | "o" -> print_char ``
                          | "O" -> print_char ``
                          | s   -> print_string s)];
();;

(* Pseudo-math mode *)

do_list (fun symbol -> def ("\\" ^ symbol) [Print ("<I>" ^ symbol ^ "</I>")])
  ["alpha";"beta";"gamma";"delta";"epsilon";"varepsilon";"zeta";"eta";
   "theta";"vartheta";"iota";"kappa";"lambda";"mu";"nu";"xi";"pi";"varpi";
   "rho";"varrho";"sigma";"varsigma";"tau";"upsilon";"phi";"varphi";
   "chi";"psi";"omega";"Gamma";"Delta";"Theta";"Lambda";"Xi";"Pi";
   "Sigma";"Upsilon";"Phi";"Psi";"Omega"];
def "\\," [];
def "\\mapsto" [Print "<tt>|-&gt;</tt>"];
();;

(* My personal macros *)

def "\\th" [Print "-th"];
def "\\st" [Print "st"];
def "\\nd" [Print "nd"];
def "\\rd" [Print "rd"];
def "\\ier" [Print "ier"];
def "\\iere" [Print "ire"];
def "\\ieme" [Print "ime"];
def "\\eme" [Print "e"];
def "\\eg" [Print "e.g. "];
def "\\ie" [Print "i.e.. "];
def "\\etal" [Print "<I>et al.</I> "];
def "\\begin{prop}" [Print "<p><strong>Proposition.</strong> <I>"];
def "\\end{prop}" [Print "</I><p>"];
def "\\comment" [Print "<I>(*"; Print_arg; Print "*)</I>"];
def "\\stringlit" [Print "\"<B>"; Print_arg; Print "</B>\""];
def "\\becomes" [Print "<tt>&lt;-</tt>"];
def "\\biling" [Print_arg; Skip_arg];
def "\\triling" [Print_arg; Skip_arg; Skip_arg];
def "\\abbrev" [Print_arg; Skip_arg];
def "\\citeurl" [Print "<A href=\""; Raw_arg print_string; Print "\">(url)</A>"];
def "\\Cplusplus" [Print "C++"];
();;

